// -*- c++ -*-
//
// $Id: chapter_10_mpi2.cc,v 1.1.1.1.2.1 2001/10/29 04:32:51 bbarrett Exp $
//
// Copyright 1997-2001, University of Notre Dame.
// Authors: Jeffrey M. Squyres, Jeremy G. Siek, Michael P. McNally, and
//          Andrew Lumsdaine
// 
// This file is part of the Notre Dame C++ bindings for MPI.
// 
// You should have received a copy of the License Agreement for the Notre
// Dame C++ bindings for MPI along with the software; see the file
// LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN 46556.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted subject to the conditions specified in the
// LICENSE file.
// 
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
// 
// Additional copyrights may follow.
// 

#include <iostream.h>
#include <mpi++.h>

void example10_2(void);
void example10_3(void);
void example10_4(void);
void example10_5(void);

// Example 10.1 - Please note foocomm is not foo_comm as on Page 273
// so it will not conflict with later uses of foo_comm.

class foocomm : public MPI::Intracomm {
public:
  void Send(const void* buf, int count, const MPI::Datatype& type,
	    int dest, int tag) const
  {
    // Class library functionality
    MPI::Intracomm::Send(buf, count, type, dest, tag);
    // More class library functionality
  }
};

int
main(int argc, char *argv[])
{
  MPI::Init(argc, argv);

  example10_2();
  
  example10_3();
  
  example10_4();

  example10_5();

  MPI::Finalize();
  return 0;
}
  
// Example 10.2

void
example10_2()
{
  MPI::Intracomm bar;
  if (bar == MPI::COMM_NULL)
    cout << "bar is MPI::COMM_NULL" << endl;
}

// Example 10.3

void
example10_3()
{
  MPI::Intracomm foo_comm, bar_comm, baz_comm;
  foo_comm = MPI::COMM_WORLD;
  bar_comm = MPI::COMM_WORLD.Dup();
  baz_comm = bar_comm;
  cout << "Is foocomm equal barcomm? " << (foo_comm == bar_comm) << endl;
  cout << "Is bazcomm equal barcomm? " << (baz_comm == bar_comm) << endl;
}

// Example 10.4 Erroneous Code

void
example10_4()
{
  MPI::Intracomm intra = MPI::COMM_WORLD.Dup();

  // This will show up as an unused variable if compiled with full
  // warnings on, but we keep it here a) because it's in Chapter 10,
  // and b) to show that it works and compiles properly

  MPI::Cartcomm cart(intra);
}

// Example 10.5

void
example10_5()
{
  MPI::Intercomm comm;
  comm = MPI::COMM_NULL;            // assign with COMM_NULL
  if (comm == MPI::COMM_NULL)       // true
    cout << "comm is NULL" << endl;
  if (MPI::COMM_NULL == comm)       // note -- a different function
    cout << "comm is still NULL" << endl;
}
